##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Legacy password managers, using now-outdated, insecure methods for hashing
"""
__docformat__ = 'restructuredtext'
import sys
from codecs import getencoder

try:
    from crypt import crypt
    from random import choice
except ImportError: # pragma: no cover
    # The crypt module is not universally available, apparently
    crypt = None

from zope.interface import implementer
from zope.password.interfaces import IMatchingPasswordManager

_encoder = getencoder("utf-8")

PY2 = sys.version_info[0] == 2


@implementer(IMatchingPasswordManager)
class CryptPasswordManager(object):
    """Crypt password manager.

    Implements a UNIX crypt(3) hashing scheme. Note that crypt is
    considered far inferior to more modern schemes such as SSHA hashing,
    and only uses the first 8 characters of a password.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.legacy import CryptPasswordManager

    >>> manager = CryptPasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password, salt="..")
    >>> encoded
    '{CRYPT}..I1I8wps4Na2'
    >>> manager.match(encoded)
    True
    >>> manager.checkPassword(encoded, password)
    True

    Note that this object fails to return bytes from the ``encodePassword``
    function on Python 3:

    >>> isinstance(encoded, str)
    True

    Unfortunately, crypt only looks at the first 8 characters, so matching
    against an 8 character password plus suffix always matches. Our test
    password (including utf-8 encoding) is exactly 8 characters long, and
    thus affixing 'wrong' to it tests as a correct password:

    >>> manager.checkPassword(encoded, password + u"wrong")
    True

    Using a completely different password is rejected as expected:

    >>> manager.checkPassword(encoded, 'completely wrong')
    False

    Using the `openssl passwd` command-line utility to encode ``secret``,
    we get ``erz50QD3gv4Dw`` as seeded hash.

    Our password manager generates the same value when seeded with the
    same salt, so we can be sure, our output is compatible with
    standard LDAP tools that also use crypt:

    >>> salt = 'er'
    >>> password = 'secret'
    >>> encoded = manager.encodePassword(password, salt)
    >>> encoded
    '{CRYPT}erz50QD3gv4Dw'

    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    >>> manager.encodePassword(password) != manager.encodePassword(password)
    True

    The manager only claims to implement CRYPT encodings, anything not
    starting with the string {CRYPT} returns False:

    >>> manager.match('{MD5}someotherhash')
    False

    """


    def encodePassword(self, password, salt=None):
        if salt is None:
            choices = ("ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                       "abcdefghijklmnopqrstuvwxyz"
                       "0123456789./")
            salt = choice(choices) + choice(choices)
        if PY2:
            # Py3: Python 2 can only handle ASCII for crypt.
            password = _encoder(password)[0]
        return '{CRYPT}%s' % crypt(password, salt)

    def checkPassword(self, encoded_password, password):
        return encoded_password == self.encodePassword(password,
                                                       encoded_password[7:9])

    def match(self, encoded_password):
        return encoded_password.startswith('{CRYPT}')


@implementer(IMatchingPasswordManager)
class MySQLPasswordManager(object):
    """A MySQL digest manager.

    This Password Manager implements the digest scheme as implemented in the
    MySQL PASSWORD function in MySQL versions before 4.1. Note that this
    method results in a very weak 16-byte hash.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.legacy import MySQLPasswordManager

    >>> manager = MySQLPasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {MYSQL}0ecd752c5097d395
    >>> manager.match(encoded)
    True
    >>> manager.match(encoded.decode())
    True
    >>> manager.checkPassword(encoded.decode(), password)
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    Using the password 'PHP & Information Security' should result in the
    hash ``379693e271cd3bd6``, according to
    http://phpsec.org/articles/2005/password-hashing.html

    Our password manager generates the same value when seeded with the same seed, so we
    can be sure, our output is compatible with MySQL versions before 4.1:

    >>> password = 'PHP & Information Security'
    >>> encoded = manager.encodePassword(password)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {MYSQL}379693e271cd3bd6

    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    The manager only claims to implement MYSQL encodings, anything not
    starting with the string {MYSQL} returns False:

    >>> manager.match('{MD5}someotherhash')
    False

    Spaces and tabs are ignored:

    >>> encoded = manager.encodePassword('\tign or ed')
    >>> print(encoded.decode())
    {MYSQL}75818366052c6a78
    >>> encoded = manager.encodePassword('ignored')
    >>> print(encoded.decode())
    {MYSQL}75818366052c6a78
    """


    def encodePassword(self, password):
        nr = 1345345333
        add = 7
        nr2 = 0x12345671
        for i in _encoder(password)[0]:
            if PY2:
                # In Python 2 bytes iterate over single-char strings.
                i = ord(i)
            if i == ord(b' ') or i == ord(b'\t'):
                continue # pragma: no cover (this is actually hit, but coverage isn't reporting it)
            nr ^= (((nr & 63) + add) * i) + (nr << 8)
            nr2 += (nr2 << 8) ^ nr
            add += i
        r0 = nr & ((1 << 31) - 1)
        r1 = nr2 & ((1 << 31) - 1)
        return ("{MYSQL}%08lx%08lx" % (r0, r1)).encode()

    def checkPassword(self, encoded_password, password):
        if not isinstance(encoded_password, bytes):
            encoded_password = encoded_password.encode('ascii')
        return encoded_password == self.encodePassword(password)

    def match(self, encoded_password):
        if not isinstance(encoded_password, bytes):
            encoded_password = encoded_password.encode('ascii')
        return encoded_password.startswith(b'{MYSQL}')



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password manager interface
"""
import zope.interface

class IPasswordManager(zope.interface.Interface):
    """Password manager utility interface.
    """

    def encodePassword(password):
        """Return encoded data for the given password

        Return encoded bytes.
        """

    def checkPassword(encoded_password, password):
        """Does the encoded password match the given password?

        Return True if they match, else False.
        """

class IMatchingPasswordManager(IPasswordManager):
    """Password manager with hash matching support"""

    def match(encoded_password):
        """Was the given data was encoded with this manager's scheme?

        Return True when the given data was encoded with the scheme
        implemented by this password manager.
        """



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password managers
"""
__docformat__ = 'restructuredtext'

from base64 import standard_b64decode
from base64 import standard_b64encode
from base64 import urlsafe_b64decode
from base64 import urlsafe_b64encode
from binascii import a2b_hex
from codecs import getencoder
from hashlib import md5, sha1
from os import urandom

import re

from hmac import compare_digest as _timing_safe_compare

try:
    import bcrypt
except ImportError: # pragma: no cover
    bcrypt = None

from zope.interface import implementer
from zope.password.interfaces import IMatchingPasswordManager

_enc = getencoder("utf-8")

def _encoder(s):
    if isinstance(s, bytes):
        return s
    return _enc(s)[0]


@implementer(IMatchingPasswordManager)
class PlainTextPasswordManager(object):
    """Plain text password manager.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.password import PlainTextPasswordManager

    >>> manager = PlainTextPasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password)
    >>> encoded == password.encode('utf-8')
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    The plain text password manager *never* claims to implement the scheme,
    because this would open a security hole, where a hash from a different
    scheme could be used as-is as a plain-text password. Authentication code
    that needs to support plain-text passwords need to explicitly check for
    plain-text password matches after all other options have been tested for:

    >>> manager.match(encoded)
    False
    """


    def encodePassword(self, password):
        password = _encoder(password)
        return password

    def checkPassword(self, encoded_password, password):
        return encoded_password == self.encodePassword(password)

    def match(self, encoded_password):
        # We always return False for PlainText because it was a) not encrypted
        # and b) matching against actual encryption methods would result in
        # the ability to authenticate with the un-encrypted hash as a password.
        # For example, you should not be able to authenticate with a literal
        # SSHA hash.
        return False

class _PrefixedPasswordManager(PlainTextPasswordManager):

    # The bytes prefix this object uses.
    _prefix = None

    def match(self, encoded_password):
        return _encoder(encoded_password).startswith(self._prefix)

class SSHAPasswordManager(_PrefixedPasswordManager):
    """SSHA password manager.

    SSHA is basically SHA1-encoding which also incorporates a salt
    into the encoded string. This way, stored passwords are more
    robust against dictionary attacks of attackers that could get
    access to lists of encoded passwords.

    SSHA is regularly used in LDAP databases and we should be
    compatible with passwords used there.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.password import SSHAPasswordManager

    >>> manager = SSHAPasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password, salt="")
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SSHA}BLTuxxVMXzouxtKVb7gLgNxzdAI=

    >>> manager.match(encoded)
    True
    >>> manager.match(encoded.decode())
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    Using the `slappasswd` utility to encode ``secret``, we get
    ``{SSHA}x3HIoiF9y6YRi/I4W1fkptbzTDiNr+9l`` as seeded hash.

    Our password manager generates the same value when seeded with the
    same salt, so we can be sure, our output is compatible with
    standard LDAP tools that also use SSHA:

    >>> from base64 import standard_b64decode
    >>> salt = standard_b64decode('ja/vZQ==')
    >>> password = 'secret'
    >>> encoded = manager.encodePassword(password, salt)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SSHA}x3HIoiF9y6YRi/I4W1fkptbzTDiNr+9l

    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    We can also pass a salt that is a text string:

    >>> salt = u'salt'
    >>> password = 'secret'
    >>> encoded = manager.encodePassword(password, salt)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SSHA}gVK8WC9YyFT1gMsQHTGCgT3sSv5zYWx0

    Because a random salt is generated, the output of encodePassword is
    different every time you call it.

    >>> manager.encodePassword(password) != manager.encodePassword(password)
    True

    The password manager should be able to cope with unicode strings for input:

    >>> passwd = u'foobar\u2211' # sigma-sign.
    >>> manager.checkPassword(manager.encodePassword(passwd), passwd)
    True
    >>> manager.checkPassword(manager.encodePassword(passwd).decode(), passwd)
    True

    The manager only claims to implement SSHA encodings, anything not starting
    with the string {SSHA} returns False:

    >>> manager.match('{MD5}someotherhash')
    False

    An older version of this manager used the urlsafe variant of the base64
    encoding (replacing / and + characters with _ and - respectively). Hashes
    encoded with the old manager are still supported:

    >>> encoded = '{SSHA}x3HIoiF9y6YRi_I4W1fkptbzTDiNr-9l'
    >>> manager.checkPassword(encoded, 'secret')
    True

    """

    _prefix = b'{SSHA}'

    def encodePassword(self, password, salt=None):
        if salt is None:
            salt = urandom(4)
        elif not isinstance(salt, bytes):
            salt = _encoder(salt)
        hash = sha1(_encoder(password))
        hash.update(salt)
        return self._prefix + standard_b64encode(hash.digest() + salt)

    def checkPassword(self, encoded_password, password):
        # standard_b64decode() cannot handle unicode input string.
        encoded_password = _encoder(encoded_password)
        encoded_password = encoded_password[6:]
        if b'_' in encoded_password or b'-' in encoded_password:
            # Encoded using old urlsafe_b64encode, re-encode
            byte_string = urlsafe_b64decode(encoded_password)
            encoded_password = standard_b64encode(byte_string)
        else:
            byte_string = standard_b64decode(encoded_password)
        salt = byte_string[20:]
        return _timing_safe_compare(encoded_password,
                                    self.encodePassword(password, salt)[6:])


class SMD5PasswordManager(_PrefixedPasswordManager):
    """SMD5 password manager.

    SMD5 is basically SMD5-encoding which also incorporates a salt
    into the encoded string. This way, stored passwords are more
    robust against dictionary attacks of attackers that could get
    access to lists of encoded passwords:

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.password import SMD5PasswordManager

    >>> manager = SMD5PasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password, salt="")
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SMD5}ht3czsRdtFmfGsAAGOVBOQ==

    >>> manager.match(encoded)
    True
    >>> manager.match(encoded.decode())
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    Using the ``slappasswd`` utility to encode ``secret``, we get
    ``{SMD5}zChC6x0tl2zr9fjvjZzKePV5KWA=`` as seeded hash.

    Our password manager generates the same value when seeded with the
    same salt, so we can be sure, our output is compatible with
    standard LDAP tools that also use SMD5:

    >>> from base64 import standard_b64decode
    >>> salt = standard_b64decode('9XkpYA==')
    >>> password = 'secret'
    >>> encoded = manager.encodePassword(password, salt)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SMD5}zChC6x0tl2zr9fjvjZzKePV5KWA=

    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    We can also pass a salt that is a text string:

    >>> salt = u'salt'
    >>> password = 'secret'
    >>> encoded = manager.encodePassword(password, salt)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SMD5}mc0uWpXVVe5747A4pKhGJXNhbHQ=

    Because a random salt is generated, the output of encodePassword is
    different every time you call it.

    >>> manager.encodePassword(password) != manager.encodePassword(password)
    True

    The password manager should be able to cope with unicode strings for
    input:

    >>> passwd = u'foobar\u2211' # sigma-sign.
    >>> manager.checkPassword(manager.encodePassword(passwd), passwd)
    True
    >>> manager.checkPassword(manager.encodePassword(passwd).decode(), passwd)
    True

    The manager only claims to implement SMD5 encodings, anything not starting
    with the string {SMD5} returns False:

    >>> manager.match('{MD5}someotherhash')
    False
    """

    _prefix = b'{SMD5}'

    def encodePassword(self, password, salt=None):
        if salt is None:
            salt = urandom(4)
        elif not isinstance(salt, bytes):
            salt = salt.encode('utf-8')
        hash = md5(_encoder(password))
        hash.update(salt)
        return self._prefix + standard_b64encode(hash.digest() + salt)

    def checkPassword(self, encoded_password, password):
        encoded_password = _encoder(encoded_password)
        byte_string = standard_b64decode(encoded_password[6:])
        salt = byte_string[16:]
        return _timing_safe_compare(encoded_password,
                                    self.encodePassword(password, salt))


class MD5PasswordManager(_PrefixedPasswordManager):
    """MD5 password manager.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.password import MD5PasswordManager

    >>> manager = MD5PasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {MD5}ht3czsRdtFmfGsAAGOVBOQ==
    >>> manager.match(encoded)
    True
    >>> manager.match(encoded.decode())
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    This password manager is compatible with other RFC 2307 MD5
    implementations. For example the output of the slappasswd command for
    a MD5 hashing of ``secret`` is ``{MD5}Xr4ilOzQ4PCOq3aQ0qbuaQ==``,
    and our implementation returns the same hash:

    >>> print(manager.encodePassword('secret').decode())
    {MD5}Xr4ilOzQ4PCOq3aQ0qbuaQ==

    The password manager should be able to cope with unicode strings for input:

    >>> passwd = u'foobar\u2211' # sigma-sign.
    >>> manager.checkPassword(manager.encodePassword(passwd), passwd)
    True
    >>> manager.checkPassword(manager.encodePassword(passwd).decode(), passwd)
    True

    A previous version of this manager also created a cosmetic salt, added
    to the start of the hash, but otherwise not used in creating the hash
    itself. Moreover, it generated the MD5 hash as a hex digest, not a base64
    encoded value and did not include the {MD5} prefix. Such hashed values are
    still supported too:

    >>> encoded = 'salt86dddccec45db4599f1ac00018e54139'
    >>> manager.checkPassword(encoded, password)
    True

    However, because the prefix is missing, the password manager cannot claim
    to implement the scheme:

    >>> manager.match(encoded)
    False
    """

    _prefix = b'{MD5}'

    def encodePassword(self, password, salt=None):
        # The salt argument only exists for backwards compatibility and is
        # ignored on purpose.
        return self._prefix + standard_b64encode(
            md5(_encoder(password)).digest())

    def checkPassword(self, encoded_password, password):
        encoded_password = _encoder(encoded_password)
        encoded = encoded_password[encoded_password.find(b'}') + 1:]
        if len(encoded) > 24:
            # Backwards compatible, hexencoded md5 and bogus salt
            encoded = standard_b64encode(a2b_hex(encoded[-32:]))
        return _timing_safe_compare(encoded, self.encodePassword(password)[5:])


class SHA1PasswordManager(_PrefixedPasswordManager):
    """SHA1 password manager.

    >>> from zope.interface.verify import verifyObject
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.password.password import SHA1PasswordManager

    >>> manager = SHA1PasswordManager()
    >>> verifyObject(IMatchingPasswordManager, manager)
    True

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password)
    >>> isinstance(encoded, bytes)
    True
    >>> print(encoded.decode())
    {SHA}BLTuxxVMXzouxtKVb7gLgNxzdAI=
    >>> manager.match(encoded)
    True
    >>> manager.match(encoded.decode())
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    This password manager is compatible with other RFC 2307 SHA
    implementations. For example the output of the slappasswd command for
    a SHA hashing of ``secret`` is ``{SHA}5en6G6MezRroT3XKqkdPOmY/BfQ=``,
    and our implementation returns the same hash:

    >>> print(manager.encodePassword('secret').decode())
    {SHA}5en6G6MezRroT3XKqkdPOmY/BfQ=

    The password manager should be able to cope with unicode strings for input:

    >>> passwd = u'foobar\u2211' # sigma-sign.
    >>> manager.checkPassword(manager.encodePassword(passwd), passwd)
    True
    >>> manager.checkPassword(manager.encodePassword(passwd).decode(), passwd)
    True

    A previous version of this manager also created a cosmetic salt, added
    to the start of the hash, but otherwise not used in creating the hash
    itself. Moreover, it generated the SHA hash as a hex digest, not a base64
    encoded value and did not include the {SHA} prefix. Such hashed values are
    still supported too:

    >>> encoded = 'salt04b4eec7154c5f3a2ec6d2956fb80b80dc737402'
    >>> manager.checkPassword(encoded, password)
    True

    However, because the prefix is missing, the password manager cannot claim
    to implement the scheme:

    >>> manager.match(encoded)
    False

    Previously, this password manager used {SHA1} as a prefix, but this was
    changed to be compatible with LDAP (RFC 2307). The old prefix is still
    supported (note the hexdigest encoding as well):

    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = '{SHA1}04b4eec7154c5f3a2ec6d2956fb80b80dc737402'
    >>> manager.match(encoded)
    True
    >>> manager.checkPassword(encoded, password)
    True
    >>> manager.checkPassword(encoded, password + u"wrong")
    False

    """

    _prefix = b'{SHA}'

    def encodePassword(self, password, salt=None):
        # The salt argument only exists for backwards compatibility and is
        # ignored on purpose.
        return self._prefix + standard_b64encode(
            sha1(_encoder(password)).digest())

    def checkPassword(self, encoded_password, password):
        encoded_password = _encoder(encoded_password)
        if self.match(encoded_password):
            encoded = encoded_password[encoded_password.find(b'}') + 1:]
            if len(encoded) > 28:
                # Backwards compatible, hexencoded sha1 and bogus salt
                encoded = standard_b64encode(a2b_hex(encoded[-40:]))
            return encoded == self.encodePassword(password)[5:]
        # Backwards compatible, hexdigest and no prefix
        encoded_password = standard_b64encode(a2b_hex(encoded_password[-40:]))
        return _timing_safe_compare(encoded_password, self.encodePassword(password)[5:])

    def match(self, encoded_password):
        encoded_password = _encoder(encoded_password)
        return encoded_password.startswith((self._prefix, b'{SHA1}'))


class BCRYPTPasswordManager(_PrefixedPasswordManager):
    """
    BCRYPT password manager.

    In addition to the passwords encoded by this class,
    this class can also recognize passwords encoded by :mod:`z3c.bcrypt`
    and properly match and check them.

    .. note:: This uses the :mod:`bcrypt` library in its
        implementation, which `only uses the first 72 characters
        <https://pypi.python.org/pypi/bcrypt/3.1.3#maximum-password-length>`_
        of the password when computing the hash.
    """

    _prefix = b'{BCRYPT}'
    # This is the same regex that z3c.bcrypt uses, via way of cryptacular
    # The $2a$ is a prefix.
    _z3c_bcrypt_syntax = re.compile(br'\$2a\$[0-9]{2}\$[./A-Za-z0-9]{53}')

    _clean_clear = staticmethod(_encoder)
    _clean_hashed = staticmethod(_encoder)

    def checkPassword(self, hashed_password, clear_password):
        """Check a *hashed_password* against a *clear_password*.

        >>> from zope.password.password import BCRYPTPasswordManager
        >>> manager = BCRYPTPasswordManager()
        >>> manager.checkPassword(b'not from here', None)
        False

        :param bytes hashed_password: The encoded password.
        :param unicode clear_password: The password to check.
        :returns: True iif hashed passwords are equal.
        :rtype: bool
        """
        if not self.match(hashed_password):
            return False
        pw_bytes = self._clean_clear(clear_password)
        pw_hash = hashed_password
        if hashed_password.startswith(self._prefix):
            pw_hash = hashed_password[len(self._prefix):]

        try:
            ok = bcrypt.checkpw(pw_bytes, pw_hash)
        except ValueError: # pragma: no cover
            # invalid salt
            ok = False
        return ok

    def encodePassword(self, password, salt=None):
        """Encode a `password`, with an optional `salt`.

        If `salt` is not provided, a unique hash will be generated
        for each invokation.

        :param password: The clear-text password.
        :type password: unicode
        :param salt: The salt to be used to hash the password.
        :rtype: str
        :returns: The encoded password as a byte-siring.
        """
        if salt is None:
            salt = bcrypt.gensalt()
        salt = self._clean_hashed(salt)
        pw = self._clean_clear(password)
        return self._prefix + bcrypt.hashpw(pw, salt=salt)

    def match(self, hashed_password):
        """Was the password hashed with this password manager?

        :param bytes hashed_password: The encoded password.
        :rtype: bool
        :returns: True iif the password was hashed with this manager.
        """
        hashed_password = _encoder(hashed_password)
        return (hashed_password.startswith(self._prefix)
                or self._z3c_bcrypt_syntax.match(hashed_password) is not None)

class BCRYPTKDFPasswordManager(_PrefixedPasswordManager):
    """
    BCRYPT KDF password manager.

    This manager converts a plain text password into a byte array.
    The password and salt values (randomly generated when the password
    is encoded) are combined and repeatedly hashed *rounds* times. The
    repeated hashing is designed to thwart discovery of the key via
    password guessing attacks. The higher the number of rounds, the
    slower each attempt will be.

    Compared to the :class:`BCRYPTPasswordManager`, this has the
    advantage of allowing tunable rounds, so as computing devices get
    more powerful making brute force attacks faster, the difficulty
    level can be raised (for newly encoded passwords).

    >>> from zope.password.password import BCRYPTKDFPasswordManager
    >>> manager = BCRYPTKDFPasswordManager()
    >>> manager.checkPassword(b'not from here', None)
    False

    Let's encode a password. We'll use the minimum acceptable number
    of rounds so that the tests run fast:

    >>> manager.rounds = 51
    >>> password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    >>> encoded = manager.encodePassword(password)
    >>> print(encoded.decode())
    {BCRYPTKDF}33...

    It checks out:

    >>> manager.checkPassword(encoded, password)
    True

    We can change the number of rounds for future encodings:

    >>> manager.rounds = 100
    >>> encoded2 = manager.encodePassword(password)
    >>> print(encoded2.decode())
    {BCRYPTKDF}64...
    >>> manager.checkPassword(encoded2, password)
    True

    And the old password still checks out:

    >>> manager.checkPassword(encoded, password)
    True
    """

    #: The number of rounds of hashing that should be applied.
    #: The higher the number, the slower it is. It should be at least
    #: 50.
    rounds = 1024

    #: The number of bytes long the encoded password will be. It must be
    #: at least 1 and no more than 512.
    keylen = 32

    _prefix = b'{BCRYPTKDF}'

    def _encode(self, password, salt, rounds, keylen):
        password = _encoder(password)

        key = bcrypt.kdf(password, salt=salt,
                         desired_key_bytes=keylen,
                         rounds=rounds)
        rounds_bytes = _encoder('%x' % rounds)
        result = (self._prefix
                  + rounds_bytes
                  + b'$'
                  + urlsafe_b64encode(salt)
                  + b'$'
                  + urlsafe_b64encode(key))
        return result

    def encodePassword(self, password):
        salt = bcrypt.gensalt()
        return self._encode(password, salt, self.rounds, self.keylen)

    def checkPassword(self, hashed_password, clear_password):
        hashed_password = _encoder(hashed_password)
        if not self.match(hashed_password):
            return False
        rounds, salt, key = hashed_password[len(self._prefix):].split(b'$')
        rounds = int(rounds, 16)

        salt = urlsafe_b64decode(salt)
        keylen = len(urlsafe_b64decode(key))
        encoded_password = self._encode(clear_password, salt, rounds, keylen)
        return _timing_safe_compare(hashed_password, encoded_password)


# Simple registry
managers = [
    ('Plain Text', PlainTextPasswordManager()),
    ('MD5', MD5PasswordManager()),
    ('SMD5', SMD5PasswordManager()),
    ('SHA1', SHA1PasswordManager()),
    ('SSHA', SSHAPasswordManager()),
]

if bcrypt is not None:
    managers.append(('BCRYPT', BCRYPTPasswordManager()))
    managers.append(('BCRYPTKDF', BCRYPTKDFPasswordManager()))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Implementation of the zpasswd script.
"""
from __future__ import print_function
import argparse
import os
import sys
from xml.sax.saxutils import quoteattr

import pkg_resources

VERSION = pkg_resources.get_distribution('zope.password').version

def main(argv=None, app_factory=None):
    """Top-level script function to create a new principals."""
    argv = sys.argv if argv is None else argv

    try:
        options = parse_args(argv)
    except SystemExit as e:
        if e.code:
            return 2
        return 0

    return run_app_with_options(options, app_factory)

def run_app_with_options(options, app_factory=None):
    app = Application if app_factory is None else app_factory
    app = app(options)
    try:
        return app.process()
    except KeyboardInterrupt:
        return 1
    except SystemExit as e:
        return e.code

class Principal(object):
    """Principal.

    >>> principal = Principal("id", u"title", u"login", b"password")
    >>> print(principal)
      <principal
        id="id"
        title="title"
        login="login"
        password="password"
        />

    >>> principal = Principal("id", u"title", u"login", b"password",
    ...     u"description", "SHA1")
    >>> print(principal)
      <principal
        id="id"
        title="title"
        login="login"
        password="password"
        description="description"
        password_manager="SHA1"
        />
    """

    def __init__(self, id, title, login, password,
                 description="", password_manager_name="Plain Text"):
        self.id = id
        self.login = login
        self.password = password
        self.title = title
        self.description = description
        self.password_manager_name = password_manager_name

    def getLines(self):
        lines = [
            '  <principal',
            '    id=%s' % quoteattr(self.id),
            '    title=%s' % quoteattr(self.title),
            '    login=%s' % quoteattr(self.login),
            '    password=%s' % quoteattr(self.password.decode())
            ]
        if self.description:
            lines.append('    description=%s' % quoteattr(self.description))
        if self.password_manager_name != "Plain Text":
            lines.append('    password_manager=%s'
                         % quoteattr(self.password_manager_name))
        lines.append('    />')
        return lines

    def __str__(self):
        return "\n".join(self.getLines())

TITLE = """
============================================
Principal information for inclusion in ZCML:
"""

ID_TITLE = """
Please choose an id for the principal.
"""

TITLE_TITLE = """
Please choose a title for the principal.
"""

LOGIN_TITLE = """
Please choose a login for the principal.
"""

PASSWORD_TITLE = """
Please provide a password for the principal.
"""

DESCRIPTION_TITLE = """
Please provide an optional description for the principal.
"""

class Application(object):

    title = TITLE
    id_title = ID_TITLE
    title_title = TITLE_TITLE
    login_title = LOGIN_TITLE
    password_title = PASSWORD_TITLE
    description_title = DESCRIPTION_TITLE

    def __init__(self, options):
        self.options = options
        self.need_blank_line = False

    def read_input_line(self, prompt):
        # The tests replace this to make sure the right things happen.
        read = raw_input if bytes is str else input
        return read(prompt)

    def read_password(self, prompt):
        # The tests replace this to make sure the right things happen.
        import getpass
        try:
            return getpass.getpass(prompt)
        except KeyboardInterrupt:
            # The cursor was left on the same line as the prompt,
            # which we don't like.  Print a blank line.
            print()
            raise

    def process(self):
        options = self.options

        destination = options.destination
        try:
            principal = self.get_principal()

            if destination is sys.stdout:
                print(self.title)
            print(principal, file=destination)
            print()
        finally:
            if destination is not sys.stdout:
                destination.close()

        return 0

    def get_principal(self):
        id = self.get_value(self.id_title, "Id: ", "Id may not be empty")
        title = self.get_value(self.title_title, "Title: ",
                               "Title may not be empty")
        login = self.get_value(self.login_title, "Login: ",
                               "Login may not be empty")
        password_manager_name, password_manager = self.get_password_manager()
        password = self.get_password()
        description = self.get_value(self.description_title, "Description: ",)

        password = password_manager.encodePassword(password)
        return Principal(id, title, login, password, description,
                         password_manager_name)

    def get_value(self, title, prompt, error=""):
        self.print_message(title)
        self.need_blank_line = True
        while True:
            value = self.read_input_line(prompt).strip()
            if not value and error:
                print(error, file=sys.stderr)
                continue
            return value

    def get_password_manager(self):
        default = 0
        self.print_message("Password manager:")
        print()
        managers = self.options.managers

        for i, (name, manager) in enumerate(managers):
            print("% i. %s" % (i + 1, name))
            if name == 'BCRYPT':
                default = i
            elif name == 'SSHA' and not default:
                default = i
        print()
        self.need_blank_line = True
        while True:
            password_manager = self.read_input_line(
                "Password Manager Number [%s]: " % (default + 1))
            if not password_manager:
                index = default
                break
            elif password_manager.isdigit():
                index = int(password_manager)
                if index > 0 and index <= len(managers):
                    index -= 1
                    break
            print("You must select a password manager", file=sys.stderr)
        print("%s password manager selected" % managers[index][0])
        return managers[index]

    def get_password(self):
        self.print_message(self.password_title)
        while True:
            password = self.read_password("Password: ")
            if not password:
                print("Password may not be empty", file=sys.stderr)
                continue
            if password != password.strip() or password.split() != [password]:
                print("Password may not contain spaces", file=sys.stderr)
                continue
            break
        again = self.read_password("Verify password: ")
        if again != password:
            print("Password not verified!", file=sys.stderr)
            sys.exit(1)
        return password

    def print_message(self, message):
        if self.need_blank_line:
            print()
            self.need_blank_line = False
        print(message)

def get_password_managers(config_path=None):
    from zope.password.password import managers as default_managers

    managers = default_managers

    if config_path:
        from zope.configuration import xmlconfig
        from zope.component import getUtilitiesFor
        from zope.password.interfaces import IPasswordManager

        print("Loading configuration...")
        xmlconfig.file(config_path)
        managers = []
        for name, manager in getUtilitiesFor(IPasswordManager):
            if name == "Plain Text":
                managers.insert(0, (name, manager))
            else:
                managers.append((name, manager))

    return managers or default_managers

def parse_args(argv):
    """Parse the command line, returning an object representing the input."""
    prog = os.path.split(os.path.realpath(argv[0]))[1]
    p = argparse.ArgumentParser(prog=prog)
    p.add_argument("-c", "--config", dest="config", metavar="FILE",
                   help=("path to the site.zcml configuration file"
                         " (more accurate but slow password managers registry creation)"))
    p.add_argument("-o", "--output", dest="destination", metavar="FILE",
                   help=("the file in which the output will be saved"
                         " (STDOUT by default)"),
                   default=sys.stdout,
                   type=argparse.FileType('w'))
    p.add_argument("--version", action="version", version=VERSION)
    options = p.parse_args(argv[1:])
    options.managers = get_password_managers(options.config)
    options.program = prog
    return options



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Vocabulary of password manager utility names

For use with zope.component and zope.schema.
"""
from zope.component import getUtilitiesFor
from zope.interface import directlyProvides
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from zope.password.interfaces import IPasswordManager

def PasswordManagerNamesVocabulary(context=None):
    """Return a vocabulary listing password manager implementations by name.
    """
    terms = []
    for name, util in getUtilitiesFor(IPasswordManager, context):
        terms.append(SimpleTerm(name))
    return SimpleVocabulary(terms)

directlyProvides(PasswordManagerNamesVocabulary, IVocabularyFactory)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for the zpasswd script.
"""
import contextlib
import doctest
import os
import sys
import tempfile
import unittest

import io

from zope.password import password, zpasswd


class TestBase(unittest.TestCase):

    stdin = None
    stdout = None
    stderr = None

    old_stderr = None
    old_stdout = None
    old_stdin = None

    def setUp(self):
        # Create a minimal site.zcml file
        with tempfile.NamedTemporaryFile(prefix="testsite",
                                         suffix=".zcml",
                                         delete=False) as f:
            f.write(
                b"""<configure xmlns="http://namespaces.zope.org/zope">
                  <include file="configure.zcml" package="zope.password" />
                </configure>
                """)
            self.config = f.name
            self.addCleanup(os.remove, f.name)

    @contextlib.contextmanager
    def patched_stdio(self, input_data=None):
        io_type = io.StringIO if bytes is not str else io.BytesIO
        self.stdout = io_type()
        self.stderr = io_type()

        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        self.old_stdin = sys.stdin
        sys.stdout = self.stdout
        sys.stderr = self.stderr

        if input_data is not None:
            self.stdin = io_type(input_data)
            sys.stdin = self.stdin

        try:
            yield
        finally:
            sys.stdout = self.old_stdout
            sys.stderr = self.old_stderr
            sys.stdin = self.old_stdin

    @contextlib.contextmanager
    def patched_getpass(self, func):
        import getpass
        orig_gp = getpass.getpass
        getpass.getpass = func

        try:
            yield
        finally:
            getpass.getpass = orig_gp


class ArgumentParsingTestCase(TestBase):

    def parse_args(self, args):
        argv = ["foo/bar.py"] + args
        with self.patched_stdio():
            options = zpasswd.parse_args(argv)

        self.assertEqual(options.program, "bar.py")
        return options

    def check_stdout_content(self, args, stderr=False):
        with self.assertRaises(SystemExit) as e:
            self.parse_args(args)

        e = e.exception
        self.assertEqual(e.code, 0)
        full = self.stdout
        empty = self.stderr
        if stderr:
            full = self.stderr
            empty = self.stdout
        self.assertTrue(full.getvalue())
        self.assertFalse(empty.getvalue())

    def test_no_arguments(self):
        options = self.parse_args([])
        self.assertTrue(options.managers)
        self.assertIs(options.destination, self.stdout)

    def test_version_long(self):
        self.check_stdout_content(["--version"],
                                  stderr=sys.version_info[0] == 2)

    def test_help_long(self):
        self.check_stdout_content(["--help"])

    def test_help_short(self):
        self.check_stdout_content(["-h"])

    def test_destination_short(self, option="-o"):
        handle, path = tempfile.mkstemp()
        os.close(handle)
        self.addCleanup(os.remove, path)
        options = self.parse_args([option, path])
        self.assertEqual(options.destination.name, path)

    def test_destination_long(self):
        self.test_destination_short("--output")

    def test_config_short(self):
        options = self.parse_args(["-c", self.config])
        self.assertTrue(options.managers)

    def test_config_long(self):
        options = self.parse_args(["--config", self.config])
        self.assertTrue(options.managers)

    def test_too_many_arguments(self):
        with self.assertRaises(SystemExit):
            self.parse_args(["--config", self.config, "extra stuff"])

        self.assertIn("unrecognized arguments",
                      self.stderr.getvalue())

    def test_main(self):
        with self.patched_stdio():
            x = zpasswd.main(['foo/bar.py', '--help'])
        self.assertEqual(x, 0)

        with self.patched_stdio():
            x = zpasswd.main(['foo/bar.py', '--no-such-argument'])
        self.assertEqual(x, 2)

class ControlledInputApplication(zpasswd.Application):

    def __init__(self, options, input_lines):
        super(ControlledInputApplication, self).__init__(options)
        self.__input = input_lines

    def read_input_line(self, prompt):
        return self.__input.pop(0)

    read_password = read_input_line

    def all_input_consumed(self):
        return not self.__input

class Options(object):

    config = None
    program = "[test-program]"
    managers = password.managers

    def __init__(self):
        self.destination = sys.stdout

class InputCollectionTestCase(TestBase):

    def createOptions(self):
        return Options()

    def _get_output(self):
        return self.stdout.getvalue()

    def _check_principal(self, expected, output=None):
        output = self._get_output()
        self.assertTrue(output)

        principal_lines = output.splitlines()[-(len(expected) + 1):-1]
        for line, expline in zip(principal_lines, expected):
            self.assertEqual(line.strip(), expline)

    def test_principal_information(self):
        apps = []
        def factory(options):
            app = ControlledInputApplication(
                options,
                ["id", u"title", u"login", u"1",
                 u"passwd", u"passwd", u"description"])
            apps.append(app)
            return app
        with self.patched_stdio():
            options = self.createOptions()
            zpasswd.run_app_with_options(options, factory)
        self.assertFalse(self.stderr.getvalue())
        self.assertTrue(apps[0].all_input_consumed())
        self._check_principal([
            '<principal',
            'id="id"',
            'title="title"',
            'login="login"',
            'password="passwd"',
            'description="description"',
            '/>'
        ])


class TestDestination(InputCollectionTestCase):

    destination = None

    def createOptions(self):
        opts = Options()
        destination = tempfile.NamedTemporaryFile(mode='w',
                                                  suffix=".test_zpasswd",
                                                  delete=False)
        self.addCleanup(os.remove, destination.name)
        self.destination = opts.destination = destination
        return opts

    def _get_output(self):
        with open(self.destination.name) as f:
            return f.read()


class TestRunAndApplication(TestBase):

    def test_keyboard_interrupt(self):
        class App(object):
            def __init__(self, options):
                self.options = options

            def process(self):
                raise KeyboardInterrupt()
        with self.patched_stdio():
            x = zpasswd.run_app_with_options(None, App)

        self.assertEqual(x, 1)

    def test_exit(self):
        class App(object):
            def __init__(self, options):
                self.options = options

            def process(self):
                raise SystemExit(42)
        with self.patched_stdio():
            x = zpasswd.run_app_with_options(None, App)

        self.assertEqual(x, 42)

        # Now via main
        parse_args = zpasswd.parse_args
        zpasswd.parse_args = lambda x: x
        try:
            x = zpasswd.main(argv=[], app_factory=App)
        finally:
            zpasswd.parse_args = parse_args

        self.assertEqual(x, 42)

    def test_read_input(self):
        with self.patched_stdio(input_data="hi there"):
            x = zpasswd.Application(None).read_input_line("")
        self.assertEqual(x, "hi there")


    def test_get_value(self):
        # No error message
        with self.patched_stdio(input_data="\n"):
            x = zpasswd.Application(None).get_value("", "")
        self.assertEqual(x, "")

        # With error message we retry
        with self.patched_stdio(input_data="\nYup"):
            x = zpasswd.Application(None).get_value("", "", error="Error")
        self.assertEqual(x, "Yup")


    def test_read_password(self):
        with self.patched_getpass(lambda _prompt: sys.stdin.read()):
            with self.patched_stdio(input_data="hi there"):
                x = zpasswd.Application(None).read_password("")
            self.assertEqual(x, "hi there")

    def test_read_password_cancel(self):
        def gp(_prompt):
            raise KeyboardInterrupt()

        with self.patched_getpass(gp):
            with self.patched_stdio(input_data="hi there"):
                with self.assertRaises(KeyboardInterrupt):
                    zpasswd.Application(None).read_password("")

        self.assertEqual(self.stdout.getvalue(), '\n')

    def test_get_passwd_empty(self):
        passwords = ['', 'abc', 'abc']
        passwords.reverse()
        def gp(_prompt):
            return passwords.pop()

        with self.patched_getpass(gp):
            with self.patched_stdio():
                x = zpasswd.Application(None).get_password()
        self.assertEqual(x, 'abc')
        self.assertEqual(self.stderr.getvalue(),
                         "Password may not be empty\n")

    def test_get_passwd_spaces(self):
        passwords = [' with spaces ', 'abc', 'abc']
        passwords.reverse()
        def gp(_prompt):
            return passwords.pop()

        with self.patched_getpass(gp):
            with self.patched_stdio():
                x = zpasswd.Application(None).get_password()
        self.assertEqual(x, 'abc')
        self.assertEqual(self.stderr.getvalue(),
                         "Password may not contain spaces\n")


    def test_get_passwd_verify_fail(self):
        passwords = ['abc', 'def']
        passwords.reverse()
        def gp(_prompt):
            return passwords.pop()

        with self.patched_getpass(gp):
            with self.patched_stdio():
                with self.assertRaises(SystemExit):
                    zpasswd.Application(None).get_password()
        self.assertEqual(self.stderr.getvalue(),
                         "Password not verified!\n")

    def test_get_password_manager_default(self):
        with self.patched_stdio(input_data='\n'):
            manager = zpasswd.Application(Options()).get_password_manager()
        self.assertEqual(manager[0], 'BCRYPT')

    def test_get_password_manager_bad(self):
        with self.patched_stdio(input_data='HI\n1'):
            manager = zpasswd.Application(Options()).get_password_manager()
        self.assertEqual(manager[0], 'Plain Text')
        self.assertEqual(self.stderr.getvalue(),
                         'You must select a password manager\n')

def test_suite():
    suite = doctest.DocTestSuite('zope.password.zpasswd')
    suite.addTest(unittest.defaultTestLoader.loadTestsFromName(__name__))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Password Managers Tests
"""
import contextlib
import doctest
import unittest

import bcrypt

from zope.component.testing import PlacelessSetup
from zope.interface.verify import verifyObject

from zope.password.interfaces import IMatchingPasswordManager

text_type = str if bytes is not str else unicode


class TestBCRYPTPasswordManager(unittest.TestCase):
    """Tests for custom zope.password password manager."""

    password = (
        u'Close \N{GREEK SMALL LETTER PHI}ncounterS 0f '
        u'tHe Erd K1nd'
    )

    def _make_one(self):
        from zope.password.password import BCRYPTPasswordManager
        return BCRYPTPasswordManager()

    @contextlib.contextmanager
    def _encode_twice(self, pw_mgr, salt1=None, salt2=None):
        enc_pw1 = pw_mgr.encodePassword(self.password, salt=salt1)
        enc_pw2 = pw_mgr.encodePassword(self.password, salt=salt2)
        yield (enc_pw1, enc_pw2)
        for enc_pw in (enc_pw1, enc_pw2):
            self.assertTrue(enc_pw.startswith(b'{BCRYPT}'))
            self.assertTrue(isinstance(enc_pw, bytes))

    def test_interface_compliance(self):
        pw_mgr = self._make_one()
        verifyObject(IMatchingPasswordManager, pw_mgr)

    def test_encodePassword_with_no_salt(self):
        pw_mgr = self._make_one()
        with self._encode_twice(pw_mgr,
                                salt1=None,
                                salt2=None) as encoded_passwords:
            self.assertNotEqual(*encoded_passwords)

    def test_encodePassword_with_same_salt(self):
        pw_mgr = self._make_one()
        salt = bcrypt.gensalt()
        with self._encode_twice(pw_mgr,
                                salt1=salt,
                                salt2=salt) as encoded_passwords:
            self.assertEqual(*encoded_passwords)

    def test_encodePassword_with_different_salts(self):
        pw_mgr = self._make_one()
        salt = bcrypt.gensalt()
        with self._encode_twice(pw_mgr,
                                salt1=salt,
                                salt2=None) as encoded_passwords:
            self.assertNotEqual(*encoded_passwords)
        with self._encode_twice(pw_mgr,
                                salt1=None,
                                salt2=salt) as encoded_passwords:
            self.assertNotEqual(*encoded_passwords)

    def test_encodePassword_with_unicode_salts(self):
        pw_mgr = self._make_one()
        salt = bcrypt.gensalt()
        # *handle* unicode salts (since all other encoding is handled)
        with self._encode_twice(pw_mgr,
                                salt1=text_type(salt, 'utf-8'),
                                salt2=salt) as encoded_passwords:
            self.assertEqual(*encoded_passwords)

    def test_checkPassword(self):
        encoded = (
            b'{BCRYPT}'
            b'$2b$12$ez4eHl6W1PfAWix5bPIbe.drdnyqjpuT1Cp0N.xcdxkAEbA7K6AHK'
        )
        pw_mgr = self._make_one()
        self.assertTrue(pw_mgr.checkPassword(encoded, self.password))
        # Mess with the hashed password, should not match
        encoded = encoded[:-1]
        self.assertFalse(pw_mgr.checkPassword(encoded, self.password))

        password = u"right \N{CYRILLIC CAPITAL LETTER A}"
        encoded = pw_mgr.encodePassword(password)
        self.assertTrue(pw_mgr.checkPassword(encoded, password))
        self.assertFalse(pw_mgr.checkPassword(encoded, password + u"wrong"))

        # Subsequently hashing the same password will produce a
        # different encoding
        encoded2 = pw_mgr.encodePassword(password)
        self.assertNotEqual(encoded2, encoded)
        self.assertFalse(pw_mgr.checkPassword(encoded2, password + u"wrong"))
        self.assertTrue(pw_mgr.checkPassword(encoded, password))
        self.assertTrue(pw_mgr.checkPassword(encoded2, password))

    def test_match(self):
        pw_mgr = self._make_one()
        self.assertFalse(pw_mgr.match(b'{SHA1}1lksd;kf;slkf;slkf'))
        self.assertTrue(pw_mgr.match(b'{BCRYPT}'))


class TestZ3cBcryptCompatible(unittest.TestCase):

    password = u"right \N{CYRILLIC CAPITAL LETTER A}"
    z3c_encoded = b'$2a$10$dzfwtSW1sFx5Q.9/8.3dzOyvIBz6xu4Y00kJWZpOrQ1eH4amFtHP6'


    def _make_one(self):
        from zope.password.password import BCRYPTPasswordManager
        return BCRYPTPasswordManager()

    def test_checkPassword(self):
        pw_mgr = self._make_one()
        self.assertTrue(pw_mgr.checkPassword(self.z3c_encoded, self.password))
        # Mess with the hashed password, should not match
        encoded = self.z3c_encoded[:-1]
        self.assertFalse(pw_mgr.checkPassword(encoded, self.password))

    def test_match(self):
        pw_mgr = self._make_one()
        self.assertTrue(pw_mgr.match(self.z3c_encoded))


class TestConfiguration(PlacelessSetup,
                        unittest.TestCase):

    def setUp(self):
        from zope.configuration import xmlconfig
        import zope.password
        xmlconfig.file('configure.zcml', zope.password)

    def test_crypt_utility_names(self):
        from zope.password.password import BCRYPTPasswordManager
        from zope.password.interfaces import IPasswordManager
        from zope import component

        self.assertIsInstance(component.getUtility(IPasswordManager, 'BCRYPT'),
                              BCRYPTPasswordManager)
        self.assertIsInstance(component.getUtility(IPasswordManager, 'bcrypt'),
                              BCRYPTPasswordManager)



def test_suite():
    from zope.component.testing import setUp, tearDown
    suite = unittest.TestSuite((
        doctest.DocTestSuite(
            'zope.password.password',
            optionflags=doctest.ELLIPSIS),
        doctest.DocTestSuite('zope.password.legacy'),
        doctest.DocTestSuite(
            'zope.password.testing',
            optionflags=doctest.ELLIPSIS,
            setUp=setUp,
            tearDown=tearDown),
    ))
    suite.addTests(unittest.defaultTestLoader.loadTestsFromName(__name__))
    return suite



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup password managers as utilities
"""
__docformat__ = "reStructuredText"

import zope.password
from zope.configuration import xmlconfig

def setUpPasswordManagers():
    """Helper function for setting up password manager utilities for tests

    >>> from zope.component import getUtility
    >>> from zope.password.interfaces import IMatchingPasswordManager
    >>> from zope.schema.interfaces import IVocabularyFactory
    >>> setUpPasswordManagers()

    >>> getUtility(IMatchingPasswordManager, 'BCRYPT')
    <zope.password.password.BCRYPTPasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'BCRYPTKDF')
    <zope.password.password.BCRYPTKDFPasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'Plain Text')
    <zope.password.password.PlainTextPasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'SSHA')
    <zope.password.password.SSHAPasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'SMD5')
    <zope.password.password.SMD5PasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'MD5')
    <zope.password.password.MD5PasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'SHA1')
    <zope.password.password.SHA1PasswordManager object at 0x...>
    >>> getUtility(IMatchingPasswordManager, 'MySQL')
    <zope.password.legacy.MySQLPasswordManager object at 0x...>

    >>> try:
    ...     import crypt
    ... except ImportError:
    ...     CryptPasswordManager = None
    ...     True
    ... else:
    ...     from zope.password.legacy import CryptPasswordManager
    ...     getUtility(IMatchingPasswordManager, 'Crypt') is not None
    True

    >>> voc = getUtility(IVocabularyFactory, 'Password Manager Names')
    >>> voc = voc(None)
    >>> voc
    <zope.schema.vocabulary.SimpleVocabulary object at 0x...>
    >>> 'SSHA' in voc
    True
    >>> 'Plain Text' in voc
    True
    >>> 'SHA1' in voc
    True
    >>> 'MD5' in voc
    True
    >>> 'SMD5' in voc
    True
    >>> 'MySQL' in voc
    True
    >>> 'BCRYPT' in voc
    True
    >>> 'BCRYPTKDF' in voc
    True

    >>> CryptPasswordManager is None or 'Crypt' in voc
    True

    """
    xmlconfig.file('configure.zcml', zope.password)
